"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useStyled = exports.defaultConfig = exports.StyledProvider = void 0;
var _colorMode = require("./core/colorMode");
var React = _interopRequireWildcard(require("react"));
var _reactNative = require("react-native");
var _propertyTokenMap = require("./propertyTokenMap");
var _utils = require("./utils");
var _createGlobalStylesWeb = require("./createGlobalStylesWeb");
var _createGlobalStyles = require("./createGlobalStyles");
var _injectInStyle = require("./injectInStyle");
var _Theme = require("./Theme");
var _useSafeLayoutEffect = require("./hooks/useSafeLayoutEffect");
var _createConfig = require("./createConfig");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
let colorModeSet = false;
let rootId = '';
const defaultConfig = exports.defaultConfig = {
  config: {},
  colorMode: 'light',
  components: {}
};
const defaultContextData = defaultConfig;
const StyledContext = /*#__PURE__*/React.createContext(defaultContextData);
const setCurrentColorMode = inputColorMode => {
  if (inputColorMode) {
    const currentColorMode = (0, _colorMode.get)();
    if (currentColorMode !== inputColorMode) {
      (0, _colorMode.set)(inputColorMode);
    }
    colorModeSet = true;
  }

  // if (inputColorMode) {
  //   set(inputColorMode === 'dark' ? 'dark' : 'light');
  //   colorModeSet = true;
  // }
};
const StyledProvider = ({
  config,
  colorMode,
  children,
  globalStyles,
  _experimentalNestedProvider,
  _experimentalSupportDynamicTheme
}) => {
  const inlineStyleMap = React.useRef({
    initialStyleInjected: false
  });
  const {
    themes
  } = (0, _Theme.useTheme)();
  const id = React.useId();
  if (rootId === '') {
    rootId = id;
  }
  const isRootProvider = rootId === id;
  const themeContextValue = React.useMemo(() => {
    if (colorMode) {
      return {
        themes: [...themes, colorMode]
      };
    }
    return {
      themes
    };
  }, [colorMode, themes]);
  inlineStyleMap.current.initialStyleInjected = false;
  // const id = React.useId();
  const currentConfig = React.useMemo(() => {
    let configWithPlatformSpecificUnits = (0, _utils.platformSpecificSpaceUnits)(config, _reactNative.Platform.OS);
    if (config !== null && config !== void 0 && config.themes) {
      Object.keys(config.themes).forEach(key => {
        configWithPlatformSpecificUnits.themes[key] = (0, _utils.platformSpecificSpaceUnits)(
        //@ts-ignore
        {
          tokens: config.themes[key]
        }, _reactNative.Platform.OS).tokens;
      });
      configWithPlatformSpecificUnits = (0, _createConfig.resolveThemes)(configWithPlatformSpecificUnits);
    }
    configWithPlatformSpecificUnits = (0, _utils.generateMergedThemeTokens)(configWithPlatformSpecificUnits);
    return configWithPlatformSpecificUnits;
  }, [config]);
  if (_reactNative.Platform.OS === 'web' && globalStyles) {
    const globalStyleInjector = (0, _createGlobalStylesWeb.createGlobalStylesWeb)(globalStyles);
    globalStyleInjector({
      ...currentConfig,
      propertyTokenMap: _propertyTokenMap.propertyTokenMap
    });
  }
  if (_reactNative.Platform.OS === 'web') {
    const cssVariables = (0, _utils.convertTokensToCssVariables)(currentConfig);
    (0, _injectInStyle.injectGlobalCssStyle)(cssVariables, 'variables', _experimentalSupportDynamicTheme);
  }
  const currentColorMode = React.useMemo(() => {
    return colorMode;
  }, [colorMode]);
  const _experimentalNestedProviderRef = React.useRef(null);
  React.useEffect(() => {
    let documentElement = null;
    if (_reactNative.Platform.OS === 'web') {
      if (_experimentalNestedProvider) {
        // write own code for nested colorMode
        documentElement = _experimentalNestedProviderRef.current;
      } else {
        documentElement = document.documentElement;
      }
    }
    // Add gs class name
    if (_reactNative.Platform.OS === 'web') {
      documentElement.classList.add(`gs`);
      if (isRootProvider) {
        if (currentColorMode) {
          var _documentElement$quer;
          (_documentElement$quer = documentElement.querySelector('body')) === null || _documentElement$quer === void 0 || _documentElement$quer.setAttribute('data-theme-id', currentColorMode);
          documentElement.classList.add(`gs-${currentColorMode}`);
        } else {
          documentElement.classList.add(`gs-light`);
        }
      }
    }
    (0, _colorMode.onChange)(currentColor => {
      // only for web
      if (_reactNative.Platform.OS === 'web' && !_experimentalNestedProvider) {
        const documentElement = document.documentElement;
        if (isRootProvider) {
          if (currentColor) {
            if (currentColor === 'dark') {
              var _documentElement$quer2;
              (_documentElement$quer2 = documentElement.querySelector('body')) === null || _documentElement$quer2 === void 0 || _documentElement$quer2.setAttribute('data-theme-id', 'dark');
              documentElement.classList.remove(`gs-light`);
            } else {
              var _documentElement$quer3;
              (_documentElement$quer3 = documentElement.querySelector('body')) === null || _documentElement$quer3 === void 0 || _documentElement$quer3.setAttribute('data-theme-id', 'light');
              documentElement.classList.remove(`gs-dark`);
            }
            documentElement.classList.add(`gs-${currentColor}`);
          }
        }
      }
    });

    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  React.useEffect(() => {
    if (isRootProvider) {
      setCurrentColorMode(currentColorMode);
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [currentColorMode]);
  (0, _useSafeLayoutEffect.useSafeLayoutEffect)(() => {
    if (_reactNative.Platform.OS === 'web' && typeof window !== 'undefined') {
      const toBeInjectedStyles = {};
      if (inlineStyleMap.current.initialStyleInjected) {
        return;
      }
      Object.keys(inlineStyleMap.current).forEach(key => {
        if (key !== 'initialStyleInjected') {
          const styles = inlineStyleMap.current[key];
          if (!toBeInjectedStyles[key]) {
            toBeInjectedStyles[key] = document.createDocumentFragment();
          }
          styles.forEach(style => {
            if (!document.getElementById(style.id)) {
              toBeInjectedStyles[key].appendChild(style);
            }
          });
        }
      });
      Object.keys(toBeInjectedStyles).forEach(key => {
        let wrapperElement = document.querySelector('#' + key);
        if (wrapperElement) {
          wrapperElement.appendChild(toBeInjectedStyles[key]);
        }
        // delete inlineStyleMap.current[key];
      });
      inlineStyleMap.current.initialStyleInjected = true;
    }
  });
  // // Set colormode for the first time
  if (!colorModeSet && isRootProvider) {
    setCurrentColorMode(currentColorMode);
  }
  const [animationDriverData, setAnimationDriverData] = React.useState();
  const globalStyleMap = (config === null || config === void 0 ? void 0 : config.globalStyle) && (0, _createGlobalStyles.createGlobalStyles)(config.globalStyle, _reactNative.Platform);
  const contextValue = React.useMemo(() => {
    const styledData = {
      config: currentConfig,
      globalStyle: globalStyleMap,
      animationDriverData,
      setAnimationDriverData,
      inlineStyleMap: inlineStyleMap.current,
      isConfigSet: true
    };
    if (_experimentalNestedProvider) {
      //@ts-ignore
      styledData._experimentalNestedProvider = _experimentalNestedProvider;
      //@ts-ignore
      styledData.colorMode = colorMode;
    }
    return styledData;
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [currentConfig, globalStyleMap, animationDriverData]);
  const providerComponent = /*#__PURE__*/React.createElement(_Theme.ThemeContext.Provider, {
    value: themeContextValue
  }, /*#__PURE__*/React.createElement(StyledContext.Provider, {
    value: contextValue
  }, children));
  if (_experimentalNestedProvider) {
    return (
      /*#__PURE__*/
      // @ts-ignore
      React.createElement(_reactNative.View, {
        ref: _experimentalNestedProviderRef
      }, providerComponent)
    );
  } else {
    return /*#__PURE__*/React.createElement(React.Fragment, null, providerComponent);
  }
};
exports.StyledProvider = StyledProvider;
const useStyled = () => React.useContext(StyledContext);
exports.useStyled = useStyled;
//# sourceMappingURL=StyledProvider.js.map